#version 460
#extension GL_NV_ray_tracing : require
#extension GL_GOOGLE_include_directive : enable

#ifndef RECURSION_DEPTH
    #define RECURSION_DEPTH 2
#endif

#include "../global_data.glsl"

layout(set = 0, binding = 0) uniform accelerationStructureNV tlas;

struct RayPayload {
	vec4 color;
	float distance;
    int depth;
    vec2 random_seed;
};

layout (location = 0) rayPayloadInNV RayPayload pay_load;
hitAttributeNV vec2 attribs;

void main()
{
    Triangle triangle = current_triangle(gl_PrimitiveID, attribs);

    // get color
    vec4 tmp = color(triangle);

    vec3 diffuse = tmp.xyz;
    float alpha = tmp.w;

    // get normal
    vec3 normal = normal(triangle);

    if (pay_load.depth < RECURSION_DEPTH) {
        vec3 origin = gl_WorldRayOriginNV + gl_WorldRayDirectionNV * gl_HitTNV;
        vec3 direction = normalize(normal + random_vec3(pay_load.random_seed));

        uint ray_flags = gl_RayFlagsOpaqueNV;
        uint cull_mask = 0xff;
        const int pay_load_index = 0;
        int hit_shader_index = 0;
        int miss_shader_index = 0;
        int sbt_record_stride = 0;
        float tmin = 0.001;
        float tmax = 10000.0;

        pay_load.depth++;

        traceNV(
            tlas,
            ray_flags,
            cull_mask,
            hit_shader_index,
            sbt_record_stride,
            miss_shader_index,
            origin,
            tmin,
            direction,
            tmax,
            pay_load_index
        );

        pay_load.depth--;

        diffuse *= 0.5 * pay_load.color.xyz;
    }

    pay_load.color = vec4(diffuse, alpha);
    pay_load.distance = gl_HitTNV;
}