use anyhow::Result;
use serde::{Deserialize, Serialize};

#[derive(Debug, Copy, Clone, PartialEq, Serialize, Deserialize)]
pub enum AbilityType {
    Projectile,
    SelfCast,
}

impl Default for AbilityType {
    fn default() -> Self {
        Self::Projectile
    }
}

impl std::str::FromStr for AbilityType {
    type Err = anyhow::Error;

    fn from_str(s: &str) -> Result<Self> {
        match s {
            "Projectile" => Ok(AbilityType::Projectile),
            "Self-Cast" => Ok(AbilityType::SelfCast),
            _ => Err(anyhow::Error::msg(format!(
                "Failed parsing AbilityType from {}",
                s
            ))),
        }
    }
}

impl std::fmt::Display for AbilityType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
        match self {
            AbilityType::Projectile => write!(f, "Projectile"),
            AbilityType::SelfCast => write!(f, "Self-Cast"),
        }
    }
}
