macro_rules! generate_stat {
    ($struct_name:ident, $data_type:ty, $display_name:expr) => {
        #[derive(Debug, Default, Clone, Copy, Serialize, Deserialize)]
        pub struct $struct_name($data_type);

        impl $struct_name {
            pub const DISPLAY_NAME: &'static str = $display_name;

            pub fn set(&mut self, v: $data_type) {
                self.0 = v;
            }

            pub fn raw(&self) -> $data_type {
                self.0
            }

            pub fn as_str(&self) -> &str {
                $display_name
            }
        }

        impl std::fmt::Display for $struct_name {
            fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
                write!(f, $display_name)
            }
        }

        impl From<$data_type> for $struct_name {
            fn from(value: $data_type) -> Self {
                $struct_name(value)
            }
        }

        impl From<&$struct_name> for $struct_name {
            fn from(value: &$struct_name) -> Self {
                value.clone()
            }
        }

        impl std::ops::Add for $struct_name {
            type Output = Self;

            fn add(self, other: $struct_name) -> Self {
                $struct_name(self.0 + other.0)
            }
        }

        impl std::ops::Sub for $struct_name {
            type Output = Self;

            fn sub(self, other: $struct_name) -> Self {
                $struct_name(self.0 - other.0)
            }
        }

        impl std::ops::Mul for $struct_name {
            type Output = Self;

            fn mul(self, other: $struct_name) -> Self {
                $struct_name(self.0 * other.0)
            }
        }

        impl std::ops::AddAssign for $struct_name {
            fn add_assign(&mut self, other: $struct_name) {
                self.0 += other.0
            }
        }

        impl std::ops::SubAssign for $struct_name {
            fn sub_assign(&mut self, other: $struct_name) {
                self.0 -= other.0
            }
        }

        impl std::ops::MulAssign for $struct_name {
            fn mul_assign(&mut self, other: $struct_name) {
                self.0 *= other.0
            }
        }

        impl std::ops::MulAssign<f32> for $struct_name {
            fn mul_assign(&mut self, other: f32) {
                self.0 = (self.0 as f32 * other) as $data_type;
            }
        }

        impl PartialOrd for $struct_name {
            fn partial_cmp(&self, other: &Self) -> Option<core::cmp::Ordering> {
                self.0.partial_cmp(&other.0)
            }
        }

        impl PartialEq for $struct_name {
            fn eq(&self, other: &Self) -> bool {
                self.0 == other.0
            }
        }

        impl Eq for $struct_name {}
    };
}

use super::attributes::Attributes;

pub trait AttributeAssociation {
    fn attribute_level(&self, attributes: &Attributes) -> u32;
}

macro_rules! associate_agility {
    ($struct_name:ident) => {
        impl AttributeAssociation for $struct_name {
            fn attribute_level(&self, attributes: &Attributes) -> u32 {
                attributes.agility().raw()
            }
        }
    };
}

macro_rules! associate_intelligence {
    ($struct_name:ident) => {
        impl AttributeAssociation for $struct_name {
            fn attribute_level(&self, attributes: &Attributes) -> u32 {
                attributes.intelligence().raw()
            }
        }
    };
}

macro_rules! associate_strength {
    ($struct_name:ident) => {
        impl AttributeAssociation for $struct_name {
            fn attribute_level(&self, attributes: &Attributes) -> u32 {
                attributes.strength().raw()
            }
        }
    };
}
