use assetpath::AssetPath;
use engine::prelude::*;

create_settings_section!(
    GeneralItemSettings,
    "Meta",
    {
        attribute_multiplier: f32,
        icon_target_width: u32,
        icon_target_height: u32,
        drop_chance: f32,
        drop_chance_reference_level: u32,
        drop_chance_elite_multiplier: f32,
        drop_chance_boss_multiplier: f32,
        jewel_level_multiplier: u32,
    }
);

create_settings_section!(
    RarityColorSettings,
    "RarityColors",
    {
        common: Color,
        uncommon: Color,
        magical: Color,
        rare: Color,
        epic: Color,
        legendary: Color,
    }
);

impl_from_rarity!(RarityColorSettings, Color);

create_settings_section!(
    RaritySlotSettings,
    "RarityStatisticSlots",
    {
        common: u32,
        uncommon: u32,
        magical: u32,
        rare: u32,
        epic: u32,
        legendary: u32,
    }
);

impl_from_rarity!(RaritySlotSettings, u32);

create_settings_section!(
    RarityDropRates,
    "RarityDropRates",
    {
        common: f32,
        uncommon: f32,
        magical: f32,
        rare: f32,
        epic: f32,
        legendary: f32,
    }
);

impl_from_rarity!(RarityDropRates, f32);

create_settings_section!(
    TypeDropRates,
    "TypeDropRates",
    {
        item: f32,
        ability_addon: f32,
        ability_book: f32,
    }
);

impl RarityDropRates {
    pub fn verify(&self) {
        let sum =
            self.common + self.uncommon + self.magical + self.rare + self.epic + self.legendary;

        assert!((sum - 1.0).abs() < 0.0001, "sum: {}", sum);
    }
}

create_settings_section!(
    PerStrengthLevelStatistics,
    "PerStrengthStats",
    {
        health: f32,
        health_regeneration: f32,
        physical_damage: u32,
        air_resistance: u32,
        fire_resistance: u32,
        water_resistance: u32,
    }
);

create_settings_section!(
    PerAgilityLevelStatistics,
    "PerAgilityStats",
    {
        armor: u32,
        critical_hit_chance: f32,
        critical_hit_damage: f32,
    }
);

create_settings_section!(
    PerIntelligenceLevelStatistics,
    "PerIntelligenceStats",
    {
        air_damage: u32,
        fire_damage: u32,
        water_damage: u32,
        mana: f32,
        mana_regeneration: f32,
    }
);

create_settings_section!(
    ItemIconPaths,
    "ItemIcons",
    {
        amulet: AssetPath,
        background: AssetPath,
        boots: AssetPath,
        chest: AssetPath,
        helmet: AssetPath,
        jewel: AssetPath,
        ring: AssetPath,
        belt: AssetPath,
        gloves: AssetPath,
        main_hand: AssetPath,
        off_hand: AssetPath,
    }
);

create_settings_section!(
    ItemPlaceHolderIconPaths,
    "ItemPlaceHolderIcons",
    {
        amulet: AssetPath,
        boots: AssetPath,
        chest: AssetPath,
        helmet: AssetPath,
        ring: AssetPath,
        belt: AssetPath,
        gloves: AssetPath,
        main_hand: AssetPath,
        off_hand: AssetPath,
    }
);

create_settings_section!(
    JewelIconPaths,
    "JewelIcons",
    {
        first: AssetPath,
        second: AssetPath,
        third: AssetPath,
        fourth: AssetPath,
    }
);

create_settings_section!(
    JewelRarityMultiplier,
    "JewelRarityMultiplier",
    {
        common: u32,
        uncommon: u32,
        magical: u32,
        rare: u32,
        epic: u32,
        legendary: u32,
    }
);

impl_from_rarity!(JewelRarityMultiplier, u32);

create_settings_container!(
    ItemSettings,
    {
        general: GeneralItemSettings,
        icon_paths: ItemIconPaths,
        icon_place_holder_paths: ItemPlaceHolderIconPaths,
        jewel_paths: JewelIconPaths,
        jewel_rarity_multiplier: JewelRarityMultiplier,
        rarity_color_settings: RarityColorSettings,
        rarity_slot_settings: RaritySlotSettings,
        rarity_drop_rates: RarityDropRates,
        type_drop_rates: TypeDropRates,
        per_strength_stats: PerStrengthLevelStatistics,
        per_agility_stats: PerAgilityLevelStatistics,
        per_intelligence_stats: PerIntelligenceLevelStatistics,
    }
);
