use anyhow::Result;
use engine::prelude::*;

use std::convert::TryFrom;

#[derive(Clone, Debug, Copy, PartialEq, Eq, Serialize, Deserialize)]
pub enum DamageType {
    Physical,
    Fire,
    Water,
    Air,
    // TODO: More
}

impl Default for DamageType {
    fn default() -> Self {
        Self::Physical
    }
}

impl std::str::FromStr for DamageType {
    type Err = anyhow::Error;

    fn from_str(s: &str) -> Result<Self> {
        match s {
            "Physical" => Ok(DamageType::Physical),
            "Fire" => Ok(DamageType::Fire),
            "Water" => Ok(DamageType::Water),
            "Air" => Ok(DamageType::Air),
            _ => {
                return Err(anyhow::Error::msg(format!(
                    "Failed parsing DamageType from {}",
                    s
                )))
            }
        }
    }
}

impl std::fmt::Display for DamageType {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> Result<(), std::fmt::Error> {
        match self {
            DamageType::Physical => write!(f, "Physical"),
            DamageType::Fire => write!(f, "Fire"),
            DamageType::Water => write!(f, "Water"),
            DamageType::Air => write!(f, "Air"),
        }
    }
}

impl Into<Color> for DamageType {
    fn into(self) -> Color {
        // when changing colors, make sure you also change them in data/gui/xml/editor/abilityeditor/damage_type.xml

        match self {
            DamageType::Physical => {
                Color::try_from("#A0522D").expect("could not convert sienna brown")
            }
            DamageType::Fire => Color::try_from("#F62817").expect("could not convert fire red"),
            DamageType::Water => Color::try_from("#105EE3").expect("could not convert blue"),
            DamageType::Air => Color::try_from("#20A6EB").expect("could not convert light blue"),
        }
    }
}
